/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.rational.report.web.core.app.internal.common.FolderWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Menu");
dojo.require("com.ibm.rational.report.web.core.app.internal.Util");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.ReportWidget");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.MessageDialog");
dojo.require("com.ibm.rational.report.web.core.client.internal.ReportClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.view.navigation", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");

var Util					= com.ibm.rational.report.web.core.app.internal.Util;
var ReportClient 			= com.ibm.rational.report.web.core.client.internal.ReportClient;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var session					= com.ibm.team.repository.web.client.session;

var NavigationMessages		= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.view.navigation", "Messages");
var CommonMessages	    	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");

var TWISTIE_OPEN_SRC		= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/grey-twistie-open.gif");
var TWISTIE_CLOSED_SRC		= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/grey-twistie-closed.gif"); 
var DROPDOWN_HOVER			= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/dd-arrow-hov.gif"); 
var DROPDOWN				= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/dd-arrow.gif");

var FOLDER_WAIT				= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/folder_wait.gif");
var FOLDER_OPEN				= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/folder_open.gif");
var FOLDER_CLOSED			= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/folder_closed.gif");
var FOLDER_OPEN_SHARED		= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/folder_open_shared.gif");
var FOLDER_CLOSED_SHARED	= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/folder_closed_shared.gif");

/**
 * Displays a report folder with its contents including sub-folders.
 */
dojo.declare(
	"com.ibm.rational.report.web.core.app.internal.common.FolderWidget",
	[dijit._Widget, dijit._Templated], {
		
	templatePath: dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/templates/FolderWidget.html"),

	NEW_FOLDER_ICON_CLASS: "newReportFolderIcon",
	UPLOAD_REPORT_DESIGN: "uploadReportDesginIcon",
	ADD_TO_FOLDER_ICON: "addToToFolderIcon",
	DELETE_ICON_CLASS: "deleteReportIcon",
	REPORT_PROPERTIES_ICON: "reportPropertiesIcon",

	postCreate: function() {
		this._folderIcon.src = FOLDER_CLOSED;
		this._dropDownImage.src = DROPDOWN;
		this._folder = this.folder;
		this._expand = this.expand;
		this._parentWidget = this.parentWidget;
		this._controller = this.controller;
		this._shared = this.folder.shared;
		this._fetchedOnce = false;
		
		this._menu = null;
		this._isMenuOpen = false;
		
		this._dropDownImage.alt = CommonMessages.OPEN_ALT_LABEL;
		
		// Store the text node in a variable so we can change it if the
		// folder gets renamed.		
		this._textNode = document.createTextNode(this._folder.name);
		
		// Insert the folder name
		this._folderLinkTd.appendChild(this._textNode);
		
		// Expand or collapse as appropriate
		if (this._expand == true) {
			this._runAction("maximize");
		} else {
			this._runAction("minimize");
		}
		
		// For access to "this" in anonymous functions
		var self = this;
		
		if (!dojo.isIE) {
			this._togglerLink.className = "toggle-mozilla";			
			this._toggler.className = "toggle-mozilla";
			this._folderIcon.className = "folder-icon-mozilla";
			this._folderLinkTd.className = "folder-label-mozilla";
			this.foldersDiv.className = "folders-tree-mozilla";
			this.reportsDiv.className = "reports-tree-mozilla";
		}
			
		// Hook the event for the triangle twisty
		dojo.connect(this._togglerLink, "onclick", null,
			function(e) {
				e.preventDefault();
				self._togglerClicked(e);
			}
		);
		
		dojo.connect(this._compactFolderTable, "onfocus", null,
			function(e) {
				e.preventDefault();
				self._dropDown.style.visibility = "visible";
				self._dropDown.style.display = "inline";
				self._dropDownImage.src = DROPDOWN;
			}
		);	
			
		dojo.connect(this._compactFolderTable, "onblur", function(e){
			e.preventDefault();
			if(!self._isMenuOpen){
				self._dropDown.style.visibility = "hidden";
				self._dropDown.style.display = "none";
				self._dropDownImage.src = DROPDOWN;
			}
		});
		
		dojo.connect(this._togglerLink, "onfocus", null,
			function(e) {
				e.preventDefault();
				self._dropDown.style.visibility = "visible";
				self._dropDown.style.display = "inline";
				self._dropDownImage.src = DROPDOWN;
			}
		);		

		dojo.connect(this._dropDown, "onfocus", null,
			function(e) {
				e.preventDefault();
				self._dropDownImage.src = DROPDOWN_HOVER;
			}
		);
		
		dojo.connect(this._dropDown, "onblur", null,
			function(e) {
				e.preventDefault();
				if(!self._isMenuOpen){
					self._dropDown.style.visibility = "hidden";
					self._dropDown.style.display = "none";
				}
			}
		);
		
		dojo.connect(document, "onkeypress", 
			function(e){
				if(e.keyCode == 27 && self._isMenuOpen){
					self._closeMenu();
					self._dropDown.focus();
				}
			}
		);
		
		if (!dojo.isIE) {
			var itemRowDivTopLeft = document.createElement("span");
			itemRowDivTopLeft.className = "itemRowDivTopLeft";
			var itemRowDivTopRight = document.createElement("span");
			itemRowDivTopRight.className = "itemRowDivTopRight";
			this._treeItem.insertBefore(itemRowDivTopRight, this._treeItem.firstChild);
			this._treeItem.insertBefore(itemRowDivTopLeft, this._treeItem.firstChild);
		}

		// Hide the drop-down indicator initially
		this._dropDown.style.visibility = "hidden";
		this._dropDown.style.display = "none";

		// Hook the events for the drop-down indicator
		dojo.connect(this._dropDownImage, "onmouseover", null,
			function(e) {
				e.preventDefault();
				self._dropDownImage.src = DROPDOWN_HOVER;
			}
		);
		dojo.connect(this._dropDownImage, "onmouseout", null,
			function(e) {
				e.preventDefault();
				self._dropDownImage.src = DROPDOWN;
			}
		);
		dojo.connect(this._dropDown, "onclick", null,
			function(e) {
				e.preventDefault();
				self._openMenu(e);
			}
		);
			
		if (!dojo.isIE) {
			var itemRowDivBottomLeft = document.createElement("span");
			itemRowDivBottomLeft.className = "itemRowDivBottomLeft";
			var itemRowDivBottomRight = document.createElement("span");
			itemRowDivBottomRight.className = "itemRowDivBottomRight";
			this._treeItem.appendChild(itemRowDivBottomLeft);
			this._treeItem.appendChild(itemRowDivBottomRight);
		}		
		
		// Blue hover support for the whole item
		dojo.connect(this._treeItem, "onmouseover", null,
			function(e) {
				e.preventDefault();
				if (self._isMenuOpen) {
					// We're already in the menu, ignore this
					self._mouseOutOccurred = false;
					return;
				}
				self._treeItem.className = "treeItemSelected";
				self._dropDown.style.visibility = "visible";
				self._dropDown.style.display = "inline";
			}
		);
		dojo.connect(this._treeItem, "onmouseout", null,
			function(e) {
				e.preventDefault();
				if (self._isMenuOpen) {
					// We're in the menu, don't remove the hover
					// but remember that we left
					self._mouseOutOccurred = true;
					return;
				}
				self._treeItem.className = "treeItem";
				self._dropDown.style.visibility = "hidden";
				self._dropDown.style.display = "none";
			}
		);
	},
	
	_updateNode: function(treeNode){
		Util.removeChildren(this._folderLinkTd);	
		this._textNode = document.createTextNode(treeNode.name);
		this._folderLinkTd.appendChild(this._textNode);
		this._folder = treeNode;
		this._shared = this._folder.shared;
		// Expand or collapse as appropriate
		if (this._expand == true) {
			this._runAction("maximize");
		} else {
			this._runAction("minimize");
		}
	},
	
	_fetchChildren: function() {
		this.currentTogglersrc = this._toggler.src;
		this._toggler.src = FOLDER_WAIT;
		var projectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
		var owner =  session.getAuthenticatedContributor();		
		var params = {projectAreaId: projectArea.itemId, ownerId: owner.itemId, parentId: this._folder.itemId};
		
		var srh = new ServiceResponseHandler(this, "_fetchChildrenSuccess", "_fetchChildrenError");				
		ReportClient.getReportTreeNodes(srh, params);
	},
	
	_fetchChildrenSuccess: function(children) {
		this._toggler.src = this.currentTogglersrc;
		var nodeMap = new Object();
			
		var folderNodes = this.foldersDiv.childNodes;		
		for(var i=0;i<folderNodes.length;i++){
			nodeMap[folderNodes[i].id] = folderNodes[i];
		}
		var reportNodes = this.reportsDiv.childNodes;
		for(var j=0;j<reportNodes.length;j++){
			nodeMap[reportNodes[j].id] = reportNodes[j];
		}
		
		for (var i in children) {
			var child = children[i];
			nodeMap[child.itemId] = null;
			var widget = dijit.byId(child.itemId);
			if(widget!=null && widget._parentWidget!=this) {
				widget.destroyRecursive(false);
			} 
			var widget = dijit.byId(child.itemId); 
			if(widget == null) {
				if (child.report != null) {
					var reportNode = new com.ibm.rational.report.web.core.app.internal.common.ReportWidget
						({  controller: this._controller, report: child, parentWidget:this, id: child.itemId});
					this.reportsDiv.appendChild(reportNode.domNode);
				} else {
					var folderNode = new com.ibm.rational.report.web.core.app.internal.common.FolderWidget
							({ controller: this._controller, folder: child, expand: false, parentWidget: this, id: child.itemId});
					this.foldersDiv.appendChild(folderNode.domNode);
				}
			}
			else{
				widget._updateNode(child);	
			}
		}
		for(var k=0;k<folderNodes.length;k++){
			var domNode = nodeMap[folderNodes[k].id];
			if(domNode !=null){
				var widget = dijit.byId(folderNodes[k].id);
				if(widget!=null) {
					widget.destroyRecursive(false);
					k--;
				}
			}			
		}
		for(var m=0;m<reportNodes.length;m++){
			var domNode = nodeMap[reportNodes[m].id];
			if(domNode !=null){
				var widget = dijit.byId(reportNodes[m].id);
				if(widget!=null) {
					widget.destroyRecursive(false);
					m--;
				}
			}			
		}
		this._fetchedOnce = true;
	},
	
	_fetchChildrenError: function(){
		//TODO: check if we can retry
		this._toggler.src = this.currentTogglersrc;
		this._fetchedOnce = false;
		this._runAction("minimize");
	},
			
	_runAction: function(action) {
		if (action === "minimize") {
			this._setToggler(TWISTIE_CLOSED_SRC, "/maximize");
			this._folderIcon.src = FOLDER_CLOSED;
			if (this._shared) {
				this._folderIcon.src = FOLDER_CLOSED_SHARED;
			}
	        this.foldersDiv.style.display = "none";
	        this.reportsDiv.style.display = "none";
			this._expand=false;
		} else if (action === "maximize") {
			this._setToggler(TWISTIE_OPEN_SRC, "/minimize");
			this._folderIcon.src = FOLDER_OPEN;
			if (this._shared) {
				this._folderIcon.src = FOLDER_OPEN_SHARED;
			}
	        this.foldersDiv.style.display = "";
	        this.reportsDiv.style.display = "";
			
			// fetch the children only never fetched before.
			//if (!this._fetchedOnce)
			this._fetchChildren();
				
			this._expand=true;
		}
	},
	
	_setToggler: function(currentStateImg, actionUri) {
		this._toggler.src = currentStateImg;
		this._togglerLink.href = actionUri;
	},
	
	_togglerClicked: function(e) {
		// extract the action we're going to take from the link's href
		// (something like "minimize", "maximize", etc.)
		var node = this._togglerLink;
		var action = node.href.slice(node.href.lastIndexOf("/") + 1);
		this._runAction(action);
	},
	
	_newFolder: function() {
		this._controller.newReportFolder(this._folder);
	},
	
	_uploadReport: function() {
		this._controller.newReportDesign(this._folder.itemId);
	},
	
	_addToFolder: function() {
		this._controller.addReportsToFolder(this._folder.itemId);
	},
	
	_deleteFromTree: function() {
		this._controller.deleteReportNode(this._folder, false);
	},
	
	_delete: function() {
		this._controller.deleteReportNode(this._folder, true);
	},
	
	_properties: function() {
		this._controller.editReportNode(this._folder, null);
	},
	
	_displayConfirmationMessage: function() {
		var self = this;
		var confirmationDialog = new com.ibm.rational.report.web.core.app.internal.common.MessageDialog(
			{title: CommonMessages.CONFIRMATION, prompt: CommonMessages.CONFIRMATION_DELETE_FOLDER, promptType: "Warning", buttonType: "YesNo"});
		dojo.connect(confirmationDialog.leftButton, "onclick", confirmationDialog.dialog,
			function (e) {
				dojo.stopEvent(e);
				self._deleteFromTree();
				confirmationDialog.close();
			}
		);	
		dojo.connect(confirmationDialog.rightButton, "onclick", confirmationDialog.dialog,
			function (e) {
				dojo.stopEvent(e);
				confirmationDialog.close();
			}
		);		
	},
	
	_openMenu: function(e) {
		this._dropDownImage.src  = DROPDOWN;//making sure that the white arrow is displayed when the menu is opened(for accessibility)
		if (this._isMenuOpen) {
			this._closeMenu();
			return;
		}
		
		if (!this._menu) {
			var editable = this._controller.secMgr.isOwner(this._folder) || this._controller.secMgr.isAdmin();
			var menu = new dijit.Menu();
			menu.addChild(new dijit.MenuItem({
				label: NavigationMessages.NEW_REPORT_FOLDER_MENU_ITEM_LABEL,
				iconClass: this.NEW_FOLDER_ICON_CLASS,
				onClick: dojo.hitch(this, this._newFolder),
				disabled: !editable
			}));
			menu.addChild(new dijit.MenuItem({
				label: NavigationMessages.UPLOAD_REPORT_DESIGN_MENU_ITEM_LABEL,
				iconClass: this.UPLOAD_REPORT_DESIGN,
				onClick: dojo.hitch(this, this._uploadReport),
				disabled: !editable
			}));
			menu.addChild(new dijit.MenuItem({
				label: NavigationMessages.ADD_REPORTS_TO_FOLDER_MENU_ITEM_LABEL,
				iconClass: this.ADD_TO_FOLDER_ICON,
				onClick: dojo.hitch(this, this._addToFolder),
				disabled: !editable
			}));
			menu.addChild(new dijit.MenuItem({
				label: NavigationMessages.DELETE_MENU_ITEM_LABEL,
				iconClass: this.DELETE_ICON_CLASS,
				onClick: dojo.hitch(this, this._displayConfirmationMessage),
				disabled: !editable
			}));
			menu.addChild(new dijit.MenuSeparator());
			menu.addChild(new dijit.MenuItem({
				label: NavigationMessages.PROPERTIES_MENU_ITEM_LABEL,
				iconClass: this.REPORT_PROPERTIES_ICON,
				onClick: dojo.hitch(this, this._properties)
			}));
			this._menu = menu;
		}
		var coords = dojo.coords(e.target, true);
		dijit.popup.open({ popup: this._menu, parent: this, x: coords.x + 10, y: coords.y + 10, onExecute: dojo.hitch(this,this._closeMenu),orient: 'BL' });
		//register event handler that causes menu to close
		this._menuCloseOnBody = this.connect(this._menu,"_onBlur",this._closeMenu);
		this._isMenuOpen = true;
		this._menu.focus();			
	},
	
	_closeMenu: function(e){
		if (e) {
			dojo.stopEvent(e);
		}
		if (this._mouseOutOccurred) {
			this._treeItem.className = "treeItem";
			this._dropDown.style.visibility = "hidden";
			this._dropDown.style.display = "none";
		}
		this.disconnect(this._menuCloseOnBody);
		dijit.popup.close(this._menu);
		this._isMenuOpen = false;
	},
	
	// TODO: Move these to a utility class
	_trim: function(string) {
		return this._trimLeft(this._trimRight(string));
	},
	
	_trimLeft: function(string) {
		return string.replace(new RegExp("^[\\s]+", "g"), "");
	},
	
	_trimRight: function(string) {
		return string.replace(new RegExp("[\\s]+$", "g"), "");
	}
	
});

})();