/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.rational.report.web.core.app.view.admin.NewUserViewer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.rational.report.web.core.client.internal.UserClient");
dojo.require("com.ibm.rational.report.web.core.app.internal.MessageFormatter");
dojo.require("com.ibm.rational.report.web.core.app.internal.Util");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.view.admin", "Messages");

var CommonMessages	    	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");
var ServiceabilityMessages	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
var CoreAdminMessages	    = dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.view.admin", "Messages");
var UserClient				= com.ibm.rational.report.web.core.client.internal.UserClient;
var MessageFormatter		= com.ibm.rational.report.web.core.app.internal.MessageFormatter;
var Util					= com.ibm.rational.report.web.core.app.internal.Util;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;

var USER_TITLE_IMG_SRC	= dojo.moduleUrl("com.ibm.rational.report.web.core", "app/internal/common/icons/user.gif");

/**
 * Displays the Edit User view.
 */
dojo.declare(
	"com.ibm.rational.report.web.core.app.view.admin.NewUserViewer",
	[dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.rational.report.web.core", "app/view/admin/templates/NewUserViewer.html"),
	
	constructor: function(args) {
		this._controller = args.controller;
	},
	
	postCreate: function() {
		Util.removeChildren(this.pageTitle);
		var pageTitleIcon = document.createElement("img");
		pageTitleIcon.src = USER_TITLE_IMG_SRC;
		this.pageTitle.appendChild(pageTitleIcon);
		this.pageTitle.appendChild(document.createTextNode(CoreAdminMessages.NEW_USER_PAGE_TITLE));
				
		Util.removeChildren(this.saveButton);
		this.saveButton.appendChild(document.createTextNode(CommonMessages.SAVE_BUTTON_LABEL));
		
		Util.removeChildren(this.closeButton);
		this.closeButton.appendChild(document.createTextNode(CommonMessages.CLOSE_BUTTON_LABEL));
		
		Util.removeChildren(this.editUserIdLabel);
		this.editUserIdLabel.appendChild(document.createTextNode(CoreAdminMessages.EDIT_USER_USER_ID_LABEL));

		Util.removeChildren(this.editUserPasswordLabel);
		this.editUserPasswordLabel.appendChild(document.createTextNode(CoreAdminMessages.EDIT_USER_PASSWORD_LABEL));

		Util.removeChildren(this.editUserConfirmPasswordLabel);
		this.editUserConfirmPasswordLabel.appendChild(document.createTextNode(CoreAdminMessages.EDIT_USER_CONFIRM_PASSWORD_LABEL));
		
		Util.removeChildren(this.editUserFirstNameLabel);
		this.editUserFirstNameLabel.appendChild(document.createTextNode(CoreAdminMessages.EDIT_USER_FIRST_NAME_LABEL));
		
		Util.removeChildren(this.editUserLastNameLabel);
		this.editUserLastNameLabel.appendChild(document.createTextNode(CoreAdminMessages.EDIT_USER_LAST_NAME_LABEL));
		
		Util.removeChildren(this.editUserScreenNameLabel);
		this.editUserScreenNameLabel.appendChild(document.createTextNode(CoreAdminMessages.EDIT_USER_SCREEN_NAME_LABEL));
		
		Util.removeChildren(this.editUserEmailLabel);
		this.editUserEmailLabel.appendChild(document.createTextNode(CoreAdminMessages.EDIT_USER_EMAIL_LABEL));
		
		Util.removeChildren(this.editUserTypeLabel);
		this.editUserTypeLabel.appendChild(document.createTextNode(CoreAdminMessages.EDIT_USER_TYPE_LABEL));
				
		this._msgFormatter = new MessageFormatter(this.messageArea);

		
		var self = this;
				
		dojo.connect(this.saveButton, "onclick",
			function(e) {
				e.preventDefault();
				self._createNewUser();
			}
		);
		
		dojo.connect(this.closeButton, "onclick",
			function(e) {
				e.preventDefault();
				self._exitPage();
			}
		);
		
		dojo.connect(this.editUserForm, "onkeyup",
			function(e) {
				e.preventDefault();
				self.validateUserInput();
			}
		);
	},
		
	_createNewUser: function() {
		Util.removeChildren(this.messageArea);
		this.saveButton.disabled = true;
		var responseHandler = new ServiceResponseHandler(
			this,
			"_exitPage",
			"_postCreateNewUserError");
		var args = {
			name: dojo.trim(this.editUserId.value),
			password: dojo.trim(this.editUserPassword.value),
			firstName: dojo.trim(this.editUserFirstName.value),
			lastName: dojo.trim(this.editUserLastName.value),
			screenName: dojo.trim(this.editUserScreenName.value),
			email: dojo.trim(this.editUserEmail.value)
		};
		UserClient.postNewUser(responseHandler, args);
	},
	
	_postCreateNewUserError: function(error) {
		var msg = "";
		this._msgFormatter.displayErrorMessage(msg, error);
	},
	
	validateUserInput: function(e) {
		if (dojo.trim(this.editUserId.value).length === 0 ||
			dojo.trim(this.editUserPassword.value).length === 0 ||	
			dojo.trim(this.editUserConfirmPassword.value).length === 0 ||			
			dojo.trim(this.editUserFirstName.value).length === 0 ||
			dojo.trim(this.editUserLastName.value).length === 0 ||
			dojo.trim(this.editUserScreenName.value).length === 0 ||
			dojo.trim(this.editUserEmail.value).length === 0) {
			this.saveButton.disabled = true;
		}
		else if (dojo.trim(this.editUserPassword.value) !=
				 dojo.trim(this.editUserConfirmPassword.value)) {
			this._msgFormatter.displayErrorMessage(ServiceabilityMessages.MISMATCH_PASSWORDS, null);
			this.saveButton.disabled = true;
		}
		else {
			this.saveButton.disabled = false;
		}
	},
	
	_exitPage: function() {
	    if (typeof this._editUserProfile != "undefined") {
			this.controller.clearMainContent(); 
		}
		else {
			this.controller.manageUsers(); 			
		}		
	}
	
});

})();